package com.kuaishou.common.netty;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * @author Li Weibo  //I believe in Spring Brother.
 *
 */
public class CompressionUtil {

    private static final int BUFF_SIZE = 4096;

    public static byte[] gzip(byte[] data) {
        try (ByteArrayOutputStream bos = new ByteArrayOutputStream(
                Math.max(BUFF_SIZE, data.length / 2));
                GZIPOutputStream out = new GZIPOutputStream(bos)) {
            out.write(data);
            out.finish();
            return bos.toByteArray();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static byte[] toByteArray(InputStream input) throws IOException {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        copy(input, output, new byte[BUFF_SIZE]);
        return output.toByteArray();
    }

    public static long copy(InputStream input, OutputStream output, byte[] buffer)
            throws IOException {
        long count = 0;
        int n;
        while (-1 != (n = input.read(buffer))) {
            output.write(buffer, 0, n);
            count += n;
        }
        return count;
    }

    public static byte[] ungzip(byte[] data) {
        //注意: ByteArrayInputStream不需要close
        try (GZIPInputStream in = new GZIPInputStream(new ByteArrayInputStream(data))) {
            return toByteArray(in);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
